#! /bin/sh
# \
exec wish${TCL} "$0" ${1+"$@"}

#
# Central Example
#
# Using Central with a GUI
#
# Stuart Cassoff
#
# Fall 2020
# Version 1.0
#
# Summer 2022
# Version 1.1
#


#
# This example shows how using Central with a gui
# can make it easier to work with options like
# -textvariable or -command, where the global
# scope as well as any namespace scope
# sometimes needs to be considered.
#

#
# This example also shows how using Central with a gui
# can make it easier to work with widget paths
# since oftentimes many widgets are created
# but few are manipulated afterwards.
#
# Widget paths saved in an Central can then be easily
# accessed by name elswehere in the program.
# 


package require central::extras

namespace eval central::extras { namespace eval examples { namespace eval gui {


namespace eval pkgconfig {
	variable pkgconfig [dict create {*}{
		name	central_example_gui
		title	{Central Example (gui)}
		descr	{Using the Central Package Browser}
		author	{Stuart Cassoff}
		when	{Summer 2022}
		version	1.1
	}]
	dict set pkgconfig license,type ISC
	dict set pkgconfig license,text [regsub -all {^\s+|\s+$|\t} {
		Copyright (c) 2020,2022 Stuart Cassoff <stwo@users.sourceforge.net>

		Permission to use, copy, modify, and distribute this software for any
		purpose with or without fee is hereby granted, provided that the above
		copyright notice and this permission notice appear in all copies.

		THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
		WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
		MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
		ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
		WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
		ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
		OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
	} {}]
	proc list {} { variable pkgconfig; return [dict keys $pkgconfig] }
	proc get {key} {
		variable pkgconfig
		if {[dict exists $pkgconfig $key]} {
			return [dict get $pkgconfig $key]
		} else {
			return -code error "key not \"$key\"known"
		}
	}
	namespace export get list
	namespace ensemble create
}; # pkgconfig


proc go {} {
	package require Tk
	namespace import ::central::central

	central discover

	set W . ; set w {}
	bind $W <Key-Escape> [list destroy $W]
	bind $W <Control-q>  [list destroy $W]

	grid [set p0 [ttk::panedwindow $w.p0 -orient horizontal]] -sticky nsew
	$p0 add [set pl [ttk::panedwindow $w.p0.pl -orient vertical]] -weight 1
	$p0 add [set pr [ttk::panedwindow $w.p0.pr -orient vertical]] -weight 1

	foreach wig [list $pl $pr] which [list l r] other [list $pr $pl] {
		bindtags $wig [linsert [bindtags $wig] end MyPane_$which]
		bind MyPane_$which <Motion> "$other sashpos 0 \[$wig sashpos\ 0]"
	}

	$pl add [set ttkbrowse [ttk::labelframe $pl.ttkbrowse -text {Central Ttk Package Browser}]]
	$pr add [set tkbrowse  [ttk::labelframe $pr.tkbrowse  -text {Central Tk Package Browser}]]
	$pl add [set ptw       [ttk::labelframe $pl.ptw       -text {Packages -> Text Widget}]]
	$pr add [set dump      [ttk::labelframe $pr.dump      -text {Text Dump}]]

	foreach wig [list $ptw $dump] {
		grid [text $wig.t -width 105 -xscrollcommand [list $wig.sh set] -yscrollcommand [list $wig.sv set]] -sticky nsew -row 0 -column 0
		grid [ttk::scrollbar $wig.sv -orient vertical   -command [list $wig.t yview]] -sticky ns -row 0 -column 1
		grid [ttk::scrollbar $wig.sh -orient horizontal -command [list $wig.t xview]] -sticky ew -row 1 -column 0
	}

	foreach ww [list $W $ttkbrowse $tkbrowse $ptw $dump] {
		grid rowconfigure    $ww 0 -weight 1
		grid columnconfigure $ww 0 -weight 1
	}

	font create PackageFont {*}[font configure TkTextFont] -size -14 -underline 1
	$ptw.t tag configure package -font PackageFont

	central CentralPackageBrowser new $ttkbrowse
	central CentralPackageBrowser new $tkbrowse tk

	central packages->textWidget $ptw.t package
	$ptw.t configure -state disabled

	$dump.t insert end [central textDump]
	$dump.t configure -state disabled

	update idletasks
	$pl sashpos 0 [$pr sashpos 0]
}

}}}; # namespace central::extras::examples::gui


if {[info exists ::argv0] && [info script] eq $::argv0} {
	central::extras::examples::gui::go {*}$::argv
}


# EOF
