#
# Central Extras - Central Coordination Bureau: Extras
#
#
# Stuart Cassoff
#
# Summer 2018
# Version 0.1
#
# Fall 2020
# Version 1.0b1
#
# Summer 2022
# Version 1.0b2
# Version 1.0b3
#


if {$::tcl_platform(engine) ne "Jim"} {

package require Tcl 8.5

}; # !Jim


namespace eval central { namespace eval extras {

proc pkgconfig {subcommand args} {
	if {$subcommand ni [list get list]} {
		return -code error "unknown subcommand \"$subcommand\": must be get, or list"
	}
	set pkgconfig [dict create {*}{
		name	central::extras
		title	{Central Extras}
		descr	{Central Coordination Bureau: Extras}
		author	{Stuart Cassoff}
		when	{Summer 2022}
		version	1.0b3
	}]
	dict set pkgconfig importfrom [namespace current]
	dict set pkgconfig license,type ISC
	dict set pkgconfig license,text [regsub -all {^\s+|\s+$|\t} {
		Copyright (c) 2018,2020,2022 Stuart Cassoff <stwo@users.sourceforge.net>

		Permission to use, copy, modify, and distribute this software for any
		purpose with or without fee is hereby granted, provided that the above
		copyright notice and this permission notice appear in all copies.

		THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
		WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
		MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
		ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
		WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
		ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
		OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
	} {}]
	return -code {*}[expr {
	    $subcommand eq "list"
		? [llength $args] == 0
		    ? [list ok [dict keys $pkgconfig]]
		    : [list error "wrong # args: should be \"[namespace current]::pkgconfig list\""]
		: $subcommand eq "get"
		    ? [llength $args] == 1
			? [dict exists $pkgconfig [set key [lindex $args 0]]]
			    ? [list ok [dict get $pkgconfig $key]]
			    : [list error "key \"$key\" not known"]
			: [list error "wrong # args: should be \"[namespace current]::pkgconfig get key\""]
		    : [list uh oh]
	}]
}; # pkgconfig


## Central Extras

proc Maxstrlen {args} {
	set max 0
	foreach str $args {
		if {[set len [string length $str]] > $max} {
			set max $len
		}
	}
	return $max
}

proc textDump {{myPackages {}}} {
	if {[llength $myPackages] == 0} { set myPackages [central packages] }
	set fPackages [list]
	foreach package $myPackages {
		set keys [central packagekeys $package]
		set max [Maxstrlen 0 {*}$keys]
		set l [list]
		foreach key $keys {
			lappend l [format "%-${max}s %s" $key [central package $package $key]]
		}
		lappend fPackages [join $l \n] {} {}
	}
	return [join $fPackages \n]
}


if {$::tcl_platform(engine) ne "Jim"} {

proc SetTextText {w text} {
	if {[set state [$w cget -state]] ne "normal"} { $w configure -state normal }
	$w delete 1.0 end
	if {$text ne ""} { $w insert end $text }
	if {$state ne "normal"} { $w configure -state $state }
}

proc packages->textWidget {w {packagetag {}} {myPackages {}}} {
	if {[llength $myPackages] == 0} { set myPackages [central packages] }
	set i 0
	foreach package $myPackages {
		if {[incr i] > 1} {
			$w insert end \n\n
		}
		$w insert end $package $packagetag \n\n
		set keys [central packagekeys $package]
		set max [Maxstrlen 0 {*}$keys]
		foreach key $keys {
			$w insert end [format "%-${max}s %s%s" $key [central package $package $key] \n]
		}
	}
}


if {[catch {package require TclOO}]} {

variable TclOO no

} else { # TclOO

variable TclOO yes

oo::class create CentralPackageBrowser {
	variable w W
	variable packagesList packagesText licenseFrame
	variable packageFields
	foreach field [list name version title author when descr license,type] { variable $field }
	variable myNs
constructor {window {toolkit {ttk}}} {
	if {$toolkit ni [list tk ttk]} { return -code error "bad toolkit \"$toolkit\": must be one of tk, or ttk" }
	namespace import ::central::central
	set packageFields [list name version title author when descr license,type license,text]
	set w [string trimright [set W $window] .]
	my CreateGui $w $toolkit
	central discover
	set myNs [central find name central::extras]
	foreach package [central packages] {
		if {"title" in [central packagekeys $package]} {
			set pkgtitle [central package $package title]
		} else {
			set pkgtitle $package
		}
		if {$toolkit eq "ttk"} {
			$packagesList insert {} end -values [list $pkgtitle $package]
		} else {
			$packagesList insert end $pkgtitle
		}
	}
	if {$toolkit eq "ttk"} {
		set item [lindex [$packagesList children {}] 0]
		$packagesList focus $item
		$packagesList selection set [list $item]
	} else {
		$packagesList selection set 0
		event generate $packagesList <<ListboxSelect>>
	}
}
destructor {
}
method CreateGui {w toolkit} {
	set k $toolkit:
	grid [$k:panedwindow $w.p0 -orient horizontal] -sticky nsew -row 0 -column 0
	$w.p0 add [$k:labelframe $w.p0.m0 -text Packages]
	$w.p0 add [$k:panedwindow $w.p0.p1 -orient vertical]
	$w.p0.p1 add [$k:labelframe $w.p0.p1.m1 -text Info]
	$w.p0.p1 add [set licenseFrame [$k:labelframe $w.p0.p1.m2 -text License]]

	if {$toolkit eq "ttk"} {
		set packagesList [$k:treeview $w.p0.m0.v0 -columns {title package} -displaycolumns {title} \
				  -selectmode browse -show {} \
				  -xscrollcommand [list $w.p0.m0.u1 set] -yscrollcommand [list $w.p0.m0.u0 set]]
	} else {
		set packagesList [$k:listbox $w.p0.m0.v0 -width 23 \
				  -xscrollcommand [list $w.p0.m0.u1 set] -yscrollcommand [list $w.p0.m0.u0 set]]
	}
	grid $packagesList -sticky nsew -row 0 -column 0
	grid [$k:scrollbar $w.p0.m0.u0 -orient vertical -command [list $w.p0.m0.v0 yview]] -sticky ns -row 0 -column 1
	grid [$k:scrollbar $w.p0.m0.u1 -orient horizontal -command [list $w.p0.m0.v0 xview]] -sticky ew -row 1 -column 0


	grid [$k:label $w.p0.p1.m1.l0  -text Title]   -sticky w -row 0 -column 0
	grid [$k:label $w.p0.p1.m1.l2  -text Author]  -sticky w -row 0 -column 2
	grid [$k:label $w.p0.p1.m1.l4  -text Name]    -sticky w -row 1 -column 0
	grid [$k:label $w.p0.p1.m1.l6  -text When]    -sticky w -row 1 -column 2
	grid [$k:label $w.p0.p1.m1.l8  -text Version] -sticky w -row 2 -column 0
	grid [$k:label $w.p0.p1.m1.l10 -text License] -sticky w -row 2 -column 2
	grid [$k:label $w.p0.p1.m1.l12 -text Descr]   -sticky w -row 3 -column 0

	set lopts {-width 20 -anchor w}
	grid [$k:label $w.p0.p1.m1.l1  -textvariable [my varname title]   {*}$lopts] -sticky w -row 0 -column 1
	grid [$k:label $w.p0.p1.m1.l3  -textvariable [my varname author]  {*}$lopts] -sticky w -row 0 -column 3
	grid [$k:label $w.p0.p1.m1.l5  -textvariable [my varname name]    {*}$lopts] -sticky w -row 1 -column 1
	grid [$k:label $w.p0.p1.m1.l7  -textvariable [my varname when]    {*}$lopts] -sticky w -row 1 -column 3
	grid [$k:label $w.p0.p1.m1.l9  -textvariable [my varname version] {*}$lopts] -sticky w -row 2 -column 1
	grid [$k:label $w.p0.p1.m1.l11 -textvariable [my varname license,type]]      -sticky w -row 2 -column 3
	grid [$k:label $w.p0.p1.m1.l13 -textvariable [my varname descr]]             -sticky w -row 3 -column 1 -columnspan 3

	grid [set packagesText [text $w.p0.p1.m2.t0 -width 20 -state disabled -height 10 -wrap none -xscrollcommand [list $w.p0.p1.m2.u3 set] -yscrollcommand [list $w.p0.p1.m2.u2 set]]] -sticky nsew -row 0 -column 0
	grid [$k:scrollbar $w.p0.p1.m2.u2 -orient vertical -command [list $w.p0.p1.m2.t0 yview]] -sticky ns -row 0 -column 1
	grid [$k:scrollbar $w.p0.p1.m2.u3 -orient horizontal -command [list $w.p0.p1.m2.t0 xview]] -sticky ew -row 1 -column 0
	grid rowconfigure $w.p0.m0 0 -weight 1
	grid rowconfigure $w.p0.p1.m2 0 -weight 1
	grid columnconfigure $w.p0.m0 0 -weight 1
	grid columnconfigure $w.p0.p1.m1 1 -weight 1
	grid columnconfigure $w.p0.p1.m2 0 -weight 1

	if {$toolkit eq "ttk"} {
		$packagesList column title -width 185
	}
	bind $packagesList <<[expr {$toolkit eq "ttk" ? "Treeview" : "Listbox"}]Select>> [list [self] showPackageInfo $toolkit]
	bind $w <Map> [list focus $packagesList]
}
method showPackageInfo {toolkit} {
	if {$toolkit eq "ttk"} {
		if {[llength [set sel [$packagesList selection]]] == 0} { return }
		set item [lindex $sel 0]
		set package [$packagesList set $item package]
	} else {
		if {[llength [set sel [$packagesList curselection]]] == 0} { return }
		set package [lindex [central packages] $sel]
	}
	set packageInfo [central package $package]
	set nData 0
	foreach field $packageFields {
		if {[set $field [my dgwd {} $packageInfo $field]] ne {}} {
			incr nData
		}
	}
	${myNs}::SetTextText $packagesText ${license,text}
	if {$nData > 0} {
		$licenseFrame configure -text License
	} else {
		$licenseFrame configure -text Info
		${myNs}::SetTextText $packagesText [${myNs}::textDump $package]
	}
}
method dgwd {def dict key args} {
	if {![dict exists $dict $key {*}$args]} { return $def }
	return [dict get $dict $key {*}$args]
}
}; # oo::class::create CentralPackageBrowser

}; # TclOO

}; # !Jim


## Setup

package require central

variable CNS [namespace parent]

namespace import ${CNS}::central


if {$::tcl_platform(engine) eq "Jim"} {

variable Cmd {}

foreach Cmd [list textDump] {
	alias ${CNS}::$Cmd [namespace current]::$Cmd
	lappend ${CNS}::CSCmds $Cmd
}

set ${CNS}::CSCmds [lsort [set ${CNS}::CSCmds]]

unset CNS Cmd

} else { # !Jim

variable CESCmds [list textDump packages->textWidget]
if {$TclOO} { lappend CESCmds CentralPackageBrowser }
variable Cmd {}
variable Map [namespace ensemble configure ${CNS}::central -map]

namespace ensemble configure ${CNS}::central -subcommands \
	[lsort [concat [namespace ensemble configure ${CNS}::central -subcommands] $CESCmds]]

foreach Cmd $CESCmds { dict set Map $Cmd [namespace current]::$Cmd }

namespace ensemble configure ${CNS}::central -map $Map

unset CESCmds CNS Cmd Map TclOO

}; # Jim/Tcl


package provide central::extras 1.0b3

}}; # namespace central::extras


# EOF
