#! /bin/sh
# \
exec tclsh${TCL} "$0" "$@"

#
# Central
#
# Stuart Cassoff
#
# Fall 2020
# Version 1.0
#


package require central

namespace eval ::central { namespace eval bin { namespace eval central {

namespace eval pkgconfig {
	variable pkgconfig [dict create {*}{
		name	central_bin
		title	{Central Bin}
		descr	{Central on the Command Line}
		author	{Stuart Cassoff}
		when	{Fall 2020}
		version	1.0
	}]
	dict set pkgconfig license,type ISC
	dict set pkgconfig license,text [regsub -all {^\s+|\s+$|\t} {
		Copyright (c) 2020 Stuart Cassoff <stwo@users.sourceforge.net>

		Permission to use, copy, modify, and distribute this software for any
		purpose with or without fee is hereby granted, provided that the above
		copyright notice and this permission notice appear in all copies.

		THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
		WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
		MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
		ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
		WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
		ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
		OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
	} {}]
	proc list {} { variable pkgconfig; return [dict keys $pkgconfig] }
	proc get {key} {
		variable pkgconfig
		if {[dict exists $pkgconfig $key]} {
			return [dict get $pkgconfig $key]
		} else {
			return -code error "key not \"$key\"known"
		}
	}
	namespace export get list
	namespace ensemble create
}; # pkgconfig

proc mpd {dict {pat {*}}} {
	set f %-[::tcl::mathfunc::max 0 {*}[lmap k [set keys [dict keys $dict $pat]] { string length $k }]]s\ \ %s
	return [lmap k $keys { format $f $k [dict get $dict $k] }]
}

proc showInfo {fn} {
	set blob [read [set f [open $fn r]]][close $f]
	if {[string first pkgconfig $blob] == -1} { return }
	set c ::central::central
	set i [interp create]
	$i eval $blob
	$i eval package require central
	$i eval $c discover
	foreach package [$i eval $c packages] {
		if {$package in [list ::central ::tcl]} { continue }
		set pkg [dict create]
		set packagekeys [$i eval $c packagekeys $package]
		puts [string cat "Package : " $package " (" $fn ")"]
		puts [string cat "Keys    : " $packagekeys]
		foreach key $packagekeys {
			if {[string match "license,*" $key]} { continue }
			dict set pkg $key [$i eval $c package $package $key]
		}
		puts [string cat " " [join [mpd $pkg] "\n "] \n]

	}
	interp delete $i
}

proc go {args} {
	foreach arg $args { showInfo $arg }
}

}}}; # ::central::bin::central


if {[info exists ::argv0] && [info script] eq $::argv0} {
	::central::bin::central::go {*}$::argv
}


# EOF
