#! /bin/sh
# \
exec tclsh${TCL} "$0" "$@"

#
# Central Example - Using the "find" subcommand
#
# Will also work with Jim
#
# Stuart Cassoff
#
# Summer 2022
# Version 1.0
#


#
# Basic usage: acquire and retrieve info.
#
# Demonstrates: API, pattern matching.
#


package require central

namespace eval central { namespace eval examples { namespace eval find {

proc pkgconfig {subcommand args} {
	if {$subcommand ni [list get list]} {
		return -code error "unknown subcommand \"$subcommand\": must be get, or list"
	}
	set pkgconfig [dict create {*}{
		name	central::examples::Find
		title	Find
		descr	{Using the "find" subcommand}
		author	{Stuart Cassoff}
		when	{Summer 2022}
		version	1.0
		example	yes
		colour	purple
	}]
	dict set pkgconfig license,type ISC
	dict set pkgconfig license,text [regsub -all {^\s+|\s+$|\t} {
		Copyright (c) 2022 Stuart Cassoff <stwo@users.sourceforge.net>

		Permission to use, copy, modify, and distribute this software for any
		purpose with or without fee is hereby granted, provided that the above
		copyright notice and this permission notice appear in all copies.

		THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
		WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
		MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
		ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
		WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
		ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
		OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
	} {}]
	return -code {*}[expr {
	    $subcommand eq "list"
		? [llength $args] == 0
		    ? [list ok [dict keys $pkgconfig]]
		    : [list error "wrong # args: should be \"[namespace current]::pkgconfig list\""]
		: $subcommand eq "get"
		    ? [llength $args] == 1
			? [dict exists $pkgconfig [set key [lindex $args 0]]]
			    ? [list ok [dict get $pkgconfig $key]]
			    : [list error "key \"$key\" not known"]
			: [list error "wrong # args: should be \"[namespace current]::pkgconfig get key\""]
		    : [list uh oh]
	}]
}; #pkgconfig


proc go {{l ""}} {
	namespace import ::central::central

	# Acquire package info
	#
	central discover


	# Find all packages with a "name" field that start with "central"
	#
	puts "\nFinding Central Packages (\[central find name central*\]) ->\n  [central find name central*]"


	# Find this package
	#
	set package [central find name central::examples::Find]

	puts "\nFinding This Package (\[central find name central::examples::Find\]) ->\n  $package"


	puts "\nPackage/Namespace ->\n  $package"


	puts "\nGetting Info ->"

	# Is this package an example?
	# This assumes an "example" boolean field is known to exist.
	#
	puts "  $package is [expr {[central package $package example] ? "an" : "not an"}] example."


	# Is Central an example?
	#
	set package [central find name central]
	puts "  $package is [expr {
		   "example" in [central packagekeys $package]
		&& [central package $package example]
		? "an" : "not an"
	}] example."


	# Another way.
	#
	set package [central find name central]
	puts "  $package is [expr {
		   [central packagekeys $package example] ne {}
		&& [central package $package example]
		? "an" : "not an"
	}] example."


	# Another way, but with this example.
	#
	set package [central find name central::examples::Find]
	puts "  $package is [expr {(
		   [central packagekeys $package example] ne {}
		&& [central package $package example]
	) ? "an" : "not an"}] example."


	if {$::tcl_platform(engine) ne "Jim"} {

	# Get exports.
	#
	set package [central find name central::examples::Find]
	puts "\n$package exports ->\n  [namespace eval $package { namespace export }]"

	}; # !Jim

}


if {$::tcl_platform(engine) ne "Jim"} {

# Pretend we're a package.
# Just for the theme of using Central with packages.
#
package ifneeded central::examples::Find 1.0 {
	package provide central::examples::Find 1.0
}

package require central::examples::Find

namespace export go

}; # !Jim


}}}; # namespace central::examples::find


if {[info exists ::argv0] && [info script] eq $::argv0} {
	central::examples::find::go {*}$::argv
}


# EOF
