#! /bin/sh
# \
exec tclsh${TCL} "$0" "$@"

#
# Central Example - Using Central #2
#
# Stuart Cassoff
#
# Fall 2020
# Version 1.0
#


#
# Basic usage: acquire and retrieve info.
#
# Demonstrates: API, pattern matching.
#
#
# Call with 'l' to also output license info as well as package info.
#


package require central

namespace eval central { namespace eval examples { namespace eval usage2 {

namespace eval pkgconfig {
	variable pkgconfig [dict create {*}{
		name	usage2
		title	Usage2
		descr	{Using Central #2}
		author	{Stuart Cassoff}
		when	{Fall 2020}
		version	1.0
		example	yes
	}]
	dict set pkgconfig license,type ISC
	dict set pkgconfig license,text [regsub -all {^\s+|\s+$|\t} {
		Copyright (c) 2020 Stuart Cassoff <stwo@users.sourceforge.net>

		Permission to use, copy, modify, and distribute this software for any
		purpose with or without fee is hereby granted, provided that the above
		copyright notice and this permission notice appear in all copies.

		THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
		WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
		MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
		ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
		WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
		ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
		OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
	} {}]
	proc list {} { variable pkgconfig; return [dict keys $pkgconfig] }
	proc get {key} {
		variable pkgconfig
		if {[dict exists $pkgconfig $key]} {
			return [dict get $pkgconfig $key]
		} else {
			return -code error "key \"$key\" not known"
		}
	}
	namespace export get list
	namespace ensemble create
}; # pkgconfig


proc go {{l ""}} {
	namespace import ::central::central

	# Acquire package info
	#
	# Either way the result is the same
	#
	if 0 {
		central discover
		set packages [central packages]
	} else {
		set packages [central discover]
	}

	puts "Packages -> $packages"

	# A package "name" is its namespace
	#
	foreach package $packages {

		puts "\nPackage/Namespace -> $package"

		# Package keys are determined by the package
		#
		set keys [central packagekeys $package]

		puts "Keys -> $keys"

		# Use [central package] to retrieve package info bits
		#
		foreach key $keys {
			if {$l ne "l" && [string match license,* $key]} { continue }
			puts [string cat $key " -> " [central package $package $key]]
		}
	}
}

}}}; # namespace central::examples::usage2


if {[info exists ::argv0] && [info script] eq $::argv0} {
	central::examples::usage2::go {*}$::argv
}


# EOF
