#! /bin/sh
# \
exec tclsh${TCL} "$0" "$@"

#
# Central Example - Using Central #3
#
# Stuart Cassoff
#
# Fall 2020
# Version 1.0
#


#
# Full package info, wildcards
#


package require central

namespace eval central { namespace eval examples { namespace eval usage3 {

namespace eval pkgconfig {
	variable pkgconfig [dict create {*}{
		name	usage3
		title	Usage3
		descr	{Using Central #3}
		author	{Stuart Cassoff}
		when	{Fall 2020}
		version	1.0
		example	yes
	}]
	dict set pkgconfig license,type ISC
	dict set pkgconfig license,text [regsub -all {^\s+|\s+$|\t} {
		Copyright (c) 2020 Stuart Cassoff <stwo@users.sourceforge.net>

		Permission to use, copy, modify, and distribute this software for any
		purpose with or without fee is hereby granted, provided that the above
		copyright notice and this permission notice appear in all copies.

		THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
		WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
		MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
		ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
		WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
		ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
		OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
	} {}]
	proc list {} { variable pkgconfig; return [dict keys $pkgconfig] }
	proc get {key} {
		variable pkgconfig
		if {[dict exists $pkgconfig $key]} {
			return [dict get $pkgconfig $key]
		} else {
			return -code error "key \"$key\" not known"
		}
	}
	namespace export get list
	namespace ensemble create
}; # pkgconfig


## Pretty-printing

proc mpd {dict {pat {*}}} {
	set f %-[::tcl::mathfunc::max 0 {*}[lmap k [set keys [lsort -dictionary [dict keys $dict $pat]]] { string length $k }]]s\ \ %s
	return [lmap k $keys { format $f $k [dict get $dict $k] }]
}
proc pd {dict {pat {*}}} {
	return [join [mpd $dict $pat] \n]
}


## Example

proc go {{l ""}} {
	namespace import ::central::central

	# Standard stuff
	#
	central discover

	# Supply only the package to obtain all
	# the keys and values of the package
	#
	puts [pd [central package ::tcl]]

	puts ""

	# Wildcards may be used to select a subset of keys
	#
	puts [pd [central package ::tcl libdir,*]]
}

}}}; # namespace central::examples::usage3


if {[info exists ::argv0] && [info script] eq $::argv0} {
	central::examples::usage3::go {*}$::argv
}


# EOF
