#
# Central - Central Coordination Bureau
#
#
# Stuart Cassoff
#
# Summer 2018
# Version 0.1
#
# Fall 2020
# Version 1.0b1
#
# Summer 2022
# Version 1.0b2
# Version 1.0b3
#


namespace eval central {

if {$::tcl_platform(engine) eq "Jim"} {

proc pkgconfig {subcommand args} {
	if {$subcommand ni [list get list]} {
		return -code error "unknown subcommand \"$subcommand\": must be get, or list"
	}
	set pkgconfig [dict create {*}{
		name	central
		title	Central
		descr	{Central Coordination Bureau}
		author	{Stuart Cassoff}
		when	{Summer 2022}
		version	1.0b3
	}]
	dict set pkgconfig importfrom [namespace current]
	dict set pkgconfig initializer Central_Init
	dict set pkgconfig license,type ISC
	dict set pkgconfig license,text [regsub -all {^\s+|\s+$|\t} {
		Copyright (c) 2018,2020,2022 Stuart Cassoff <stwo@users.sourceforge.net>

		Permission to use, copy, modify, and distribute this software for any
		purpose with or without fee is hereby granted, provided that the above
		copyright notice and this permission notice appear in all copies.

		THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
		WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
		MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
		ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
		WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
		ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
		OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
	} {}]
	return -code {*}[expr {
	    $subcommand eq "list"
		? [llength $args] == 0
		    ? [list ok [dict keys $pkgconfig]]
		    : [list error "wrong # args: should be \"[namespace current]::pkgconfig list\""]
		: $subcommand eq "get"
		    ? [llength $args] == 1
			? [dict exists $pkgconfig [set key [lindex $args 0]]]
			    ? [list ok [dict get $pkgconfig $key]]
			    : [list error "key \"$key\" not known"]
			: [list error "wrong # args: should be \"[namespace current]::pkgconfig get key\""]
		    : [list uh oh]
	}]
}; # pkgconfig

} else { # !Jim

package require Tcl 8.5

namespace eval pkgconfig {
	variable pkgconfig [dict create {*}{
		name	central
		title	Central
		descr	{Central Coordination Bureau}
		author	{Stuart Cassoff}
		when	{Summer 2022}
		version	1.0b3
	}]
	dict set pkgconfig importfrom [namespace parent]
	dict set pkgconfig initializer Central_Init
	dict set pkgconfig license,type ISC
	dict set pkgconfig license,text [regsub -all {^\s+|\s+$|\t} {
		Copyright (c) 2018,2020,2022 Stuart Cassoff <stwo@users.sourceforge.net>

		Permission to use, copy, modify, and distribute this software for any
		purpose with or without fee is hereby granted, provided that the above
		copyright notice and this permission notice appear in all copies.

		THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
		WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
		MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
		ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
		WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
		ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
		OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
	} {}]
	proc list {} { variable pkgconfig; return [dict keys $pkgconfig] }
	proc get {key} {
		variable pkgconfig
		if {[dict exists $pkgconfig $key]} {
			return [dict get $pkgconfig $key]
		} else {
			return -code error "key \"$key\" not known"
		}
	}
	namespace export get list
	namespace ensemble create
}; # pkgconfig

}; # Jim/Tcl


## Central

proc IsGlobPat {p} { regexp {[*?[\]]} $p }


if {$::tcl_platform(engine) eq "Jim"} {

proc RnsSepArge {} { list "::" "::" "?arg...?" }

} else { # !Jim

proc RnsSepArge {} [list list ""  " " \
	[expr {[info tclversion] eq "8.5" ? "..." : "?arg ...?"}]]

}; # Jim/Tcl


variable Packages [list]


proc packages {} {
	variable Packages
	return $Packages
}

proc packagekeys {package {key {*}}} {
	if {$package ni [packages]} {
		return -code error "package \"$package\" not known"
	}
	return [lsearch -glob -inline -all [${package}::pkgconfig list] $key]
}

proc package {{package {*}} {key {*}}} {
	set pkgexact [expr {![IsGlobPat $package]}]
	set keyexact [expr {![IsGlobPat $key]}]
	if {$pkgexact} {
		if {$package ni [packages]} {
			return -code error "package \"$package\" not known"
		}
		if {$keyexact} {
			if {$key ni [packagekeys $package]} {
				return -code error "key \"$key\" not known"
			}
			return [${package}::pkgconfig get $key]
		}
	}
	set res [dict create]
	foreach pkg [lsearch -glob -inline -all [packages] $package] {
		foreach k [lsearch -glob -inline -all [packagekeys $pkg] $key] {
			dict set res $pkg $k [${pkg}::pkgconfig get $k]
		}
	}
	if {!$pkgexact || [dict size $res] == 0} {
		return $res
	} else {
		return [dict get $res $package]
	}
}

proc find {{key {*}} {value {*}}} {
	set glob [expr {[IsGlobPat $key] || [IsGlobPat $value]}]
	set found [list]
	foreach pkg [central packages] {
		foreach k [lsearch -glob -inline -all [central packagekeys $pkg] $key] {
			if {[string match $value [set val [central package $pkg $k]]]} {
				lappend found [list $pkg $k $val]
			}
		}
	}
	if {$glob || [llength $found] > 1} {
		return $found
	} else {
		return [lindex $found 0 0]
	}
}

proc centralized {tclpackage} {
	set none [list {} {}]
	set package [find "name" $tclpackage]
	if {$package eq {}} { return $none }
	if {"importfrom" ni [packagekeys $package]} { return $none }
	return [list $package [package $package importfrom]]
}

proc exports {tclpackage} {
	lassign [centralized $tclpackage] package importfrom
	if {$package eq {} || $importfrom eq {}} { return {} }
	return [namespace eval $importfrom { namespace export }]
}

proc import {args} {
	set force ""
	if {[llength $args] > 0 && [lindex $args 0] eq "-force"} {
		set args [lassign $args force]
	}
	if {[llength $args] == 0} {
		lassign [RnsSepArge] rns sep
		return -code error "wrong # args: should be \"${rns}central${sep}import ?-force? tclpackage ?pattern ...?\""
	}
	set args [lassign $args tclpackage]
	if {[llength $args] == 0} {
		set args [list *]
	}
	lassign [centralized $tclpackage] package importfrom
	if {$package eq {} || $importfrom eq {}} {
		return [list]
	}
	set toimport ${importfrom}::[join $args " ${importfrom}::"]
	uplevel 1 [list namespace import {*}$force {*}$toimport]
	return $toimport
}

proc initializer {tclpackage} {
	set package [find "name" $tclpackage]
	if {$package eq {}} { return {} }
	if {"initializer" ni [packagekeys $package]} { return {} }
	set init [package $package initializer]
	if {[string index $init 0] eq ":"} {
		return $init
	} elseif {"importfrom" ni [packagekeys $package]} {
		return {}
	} else {
		return [package $package importfrom]::$init
	}
}

proc initialize {tclpackage args} {
	set package [find "name" $tclpackage]
	if {$package eq {}} { return -code error "package \"$tclpackage\" not known" }
	set code error
	set res "Tcl package \"$tclpackage\" doesn't have an initialization command"
	if {"initializer" ni [packagekeys $package]} { return -code $code $res }
	set init [initializer $tclpackage]
	if {$init ne {}} {
		set code [expr {[catch {uplevel 1 [linsert $args 0 $init]} res] ? "error" : "ok"}]
	}
	return -code $code $res
}

proc require {args} {
	if {[llength $args] == 0} {
		lassign [RnsSepArge] rns sep
		set    msg "wrong # args: should be \"${rns}central${sep}require ?-exact?"
		append msg " ?-noinit? ?-noimport? ?-import {command ...}? ?-init command? -allinfo"
		append msg " tclpackage ?requirement ...?\""
		return -code error $msg
	}
	set exact ""
	set tclpackage ""
	set requirements [list]
	set init yes
	set import yes
	set initter {}
	set importing [list]
	set allinfo no
	set tclpackage_set no
	set state opt
	foreach arg $args {
		switch -exact -- $state {
		opt {
			switch -exact -- $arg {
			-exact    { set exact   -exact }
			-noinit   { set init    no     }
			-noimport { set import  no     }
			-allinfo  { set allinfo yes    }
			-init - -import { set state $arg }
			default { if {$tclpackage_set} {
					lappend requirements $arg
				  } else {
					set tclpackage $arg
					set tclpackage_set yes
				  }}
			}; #switch
		}
		-init   { set state opt; set initter   $arg }
		-import { set state opt; set importing $arg }
		}; #switch
	}
	set reqRes [uplevel 1 [list package require {*}$exact {*}$tclpackage {*}$requirements]]
	uplevel 1 [list [namespace current]::central discover]
	if {$init}   { set initRes   [uplevel 1 [list {*}$initter [initializer $tclpackage]]]
	} else       { set initRes   {} }
	if {$import} { set importRes [uplevel 1 [list [namespace current]::central import $tclpackage {*}$importing]]
	} else       { set importRes {} }
	if {$allinfo} {
		return [dict create require $reqRes init $initRes import $importRes]
	} else {
		return $reqRes
	}
}

if {$::tcl_platform(engine) eq "Jim"} {

proc discover {{exclude {}}} {
	variable Packages
	set pkgs [list]
	foreach p [info commands ::*::pkgconfig] {
		set ns [namespace parent $p]
		if {$ns ni $exclude} {
			lappend pkgs $ns
		}
	}
	set Packages [lsort $pkgs]
	return $Packages
}

} else { # !Jim

proc discover {{exclude {}}} {
	variable Packages
	set pkgs [list]
	set nsl [list ::]
	while {[llength $nsl] > 0} {
		set nsl [lassign $nsl[set nsl {}] ns]
		if {$ns in $exclude} continue
		if {[llength [info commands ${ns}::pkgconfig]] == 1} {
			lappend pkgs $ns
		}
		lappend nsl {*}[namespace children $ns]
	}
	set Packages [lsort $pkgs]
	return $Packages
}

}; # Jim/Tcl


## Setup

proc Central_Init {} { return 1.0b3 }; # For testing and demonstration


variable CSCmds [list centralized discover exports find import initialize initializer package packagekeys packages require]


if {$::tcl_platform(engine) eq "Jim"} {

proc central {cmd args} {
	variable CSCmds
	if {$cmd in $CSCmds} {
		tailcall ::central::$cmd {*}$args
	} else {
		return -code error "unknown subcommand \"$cmd\": must be [join [lrange $CSCmds 0 end-1] ", "], or [lindex $CSCmds end]"
	}
}

} else { # !Jim

namespace ensemble create -prefixes no -command central -subcommands $CSCmds

unset CSCmds

}; # Jim/Tcl


namespace export central

# Careful!
::package provide central 1.0b3


}; # namespace eval central


# EOF
